/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;


namespace ShopClassLibrary
{
	/// <summary>
	/// Summary description for interface ICustomer.
	/// </summary>
	public interface ICustomer
	{
		DataSet GetByID(string user, string pass, long clientID);

		DataSet FindByName(string user, string pass, string namePattern);

		long Add(string user, string pass, 
			     string name, string address, string phone, 
				 string fax, string eMail, out ShopStatusEnum status);

		ShopStatusEnum Update(string user, string pass, 
			                  long custumerID, string name, string address, 
							  string phone, string fax, string eMail);
	}

	/// <summary>
	/// Summary description for Customer.
	/// </summary>
	public class Customer : ICustomer
	{
		public Customer()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		public DataSet GetByID(string user, string pass, long clientID)
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				ShopStatusEnum status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status != ShopStatusEnum.OK)
					return null;

				//
				// efectuar pesquisa
				//
				ds = UtilDB.GetByID(conn, null, "Customers", "CustomerID", clientID);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetByID> da classe 'Customer': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		public DataSet FindByName(string user, string pass, string namePattern)
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				ShopStatusEnum status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status != ShopStatusEnum.OK)
					return null;
			
				//
				// efectuar pesquisa
				//
				ds = UtilDB.FindByStringField(conn, "Customers", "Name", namePattern);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <FindByName> da classe 'Customer': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}
			
			return ds;
		}

		public long Add(string user, string pass, 
			            string name, string address, string phone, string fax, 
			            string eMail, out ShopStatusEnum status)
		{
			long customerID = -1;
			OleDbConnection conn = null;

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status!= ShopStatusEnum.OK)
					return -1;

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "INSERT INTO Customers (Name, Address, PhoneNb, FaxNb, EMail) Values (?, ?, ?, ?, ?)";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("name", name);
				if (address.Length == 0)
					cmd.Parameters.Add("address", DBNull.Value);
				else
					cmd.Parameters.Add("address", address);
				if (phone.Length == 0)
					cmd.Parameters.Add("phone", DBNull.Value);
				else
					cmd.Parameters.Add("phone", phone);
				if (fax.Length == 0)
					cmd.Parameters.Add("fax", DBNull.Value);
				else
					cmd.Parameters.Add("fax", fax);
				if (eMail.Length == 0)
					cmd.Parameters.Add("email", DBNull.Value);
				else
					cmd.Parameters.Add("email", eMail);
		
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = ShopStatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					// obter novo cdigo de id gerado pela BD
					//
					OleDbCommand idCmd = new OleDbCommand("SELECT @@IDENTITY", conn);
					customerID = (int)idCmd.ExecuteScalar();
					status = ShopStatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <AddCustomer> da classe 'Customer': " + ex.Message);
				status = ShopStatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return customerID;
		}

		public ShopStatusEnum Update(string user, string pass, 
			                         long customerID, string name, string address, 
									 string phone, string fax, string eMail)
		{
			OleDbConnection conn = null;
			ShopStatusEnum status;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status != ShopStatusEnum.OK)
					return status;

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "Update Customers SET Name = ?, Address = ?, PhoneNb = ?, FaxNb = ?, EMail = ? WHERE CustomerID = ?";
				OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn);
				cmd.Parameters.Add("name", name);
				if (address.Length == 0)
					cmd.Parameters.Add("address", DBNull.Value);
				else
					cmd.Parameters.Add("address", address);
				if (phone.Length == 0)
					cmd.Parameters.Add("phone", DBNull.Value);
				else
					cmd.Parameters.Add("phone", phone);
				if (fax.Length == 0)
					cmd.Parameters.Add("fax", DBNull.Value);
				else
					cmd.Parameters.Add("fax", fax);
				if (eMail.Length == 0)
					cmd.Parameters.Add("email", DBNull.Value);
				else
					cmd.Parameters.Add("email", eMail);
				cmd.Parameters.Add("custid", customerID);

				//
				// executar o comando 
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = ShopStatusEnum.NOT_OK;
				else
					// actualizou registo
					status = ShopStatusEnum.OK;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Update> da classe 'Customer': " + ex.Message);
				status = ShopStatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}
	}
}
