/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.OleDb;

namespace ShopClassLibrary
{
	public interface IProduct
	{
		DataSet GetByID(string User, string Pass, int ProductID);
		DataSet FindByName(string User, string Pass, string NamePattern);
		int GetStock(string User, string Pass, int ProductID, out ShopStatusEnum status);
		ShopStatusEnum InStock(string User, string Pass, int ProductID, int Quant);
		ShopStatusEnum AddStock(string User, string Pass, int ProductID, int Quant);
	}

	/// <summary>
	/// Summary description for Product.
	/// </summary>
	public class Product : IProduct
	{
		#region IProduct
		/// <summary>
		/// pesquisa um determinado produto por cdigo de produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>a informao do produto</returns>
		public DataSet GetByID(string User, string Pass, int ProductID)
		{
			OleDbConnection conn = null;
			DataSet rs = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				if( UtilDB.ValidateUser(conn, null, User, Pass) != ShopStatusEnum.OK )
					return null;

				//
				// efectuar pesquisa
				//
				rs = UtilDB.GetByID(conn, null, "Products", "ProductID", ProductID);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetById> da classe 'Product': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return rs;
		}

		/// <summary>
		/// pesquisa produtos por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>informao dos produtos que obedeam ao critrio de busca</returns>
		public DataSet FindByName(string User, string Pass, string NamePattern)
		{
			OleDbConnection conn = null;
			DataSet rs = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				if( UtilDB.ValidateUser(conn, null, User, Pass) != ShopStatusEnum.OK )
					return null;

				//
				// efectuar a pesquisa
				//
				rs = UtilDB.FindByStringField(conn, "Products", "Description", NamePattern);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <FindByName> da classe 'Product': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return rs;
		}

		/// <summary>
		/// devolve a quantidade em stock de um determinado produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="status"></param>
		/// <returns></returns>
		public int GetStock(string User, string Pass, int ProductID, out ShopStatusEnum status)
		{
			OleDbConnection conn = null;
			int qt;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.OK )
					return -1;

				//
				// preparar comando
				//
				OleDbCommand cmd = new OleDbCommand("SELECT StockQtd FROM Products WHERE ProductId=?", conn);
				cmd.Parameters.Add("pid", ProductID);

				//
				// executar comando
				//
				qt = (int)cmd.ExecuteScalar();

				status = ShopStatusEnum.OK;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <InStock> da classe 'Product': " + ex.Message);
				status = ShopStatusEnum.NOT_OK;
				return -1;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return qt;
		}

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem 
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
		public ShopStatusEnum InStock(string User, string Pass, int ProductID, int Quant)
		{
			OleDbConnection conn = null;
			ShopStatusEnum status;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.OK )
					return status;

				//
				// invocar mtodo auxiliar
				//
				status = InStock(conn, null, ProductID, Quant);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <InStock> da classe 'Product': " + ex.Message);
				return ShopStatusEnum.NOT_OK;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		public ShopStatusEnum AddStock(string User, string Pass, int ProductID, int Quant)
		{
			OleDbConnection conn = null;
			ShopStatusEnum status;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.OK)
					return status;

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "Update Products SET StockQtd = StockQtd + ? WHERE ProductID = " + ProductID.ToString(); 
				OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn);
				cmd.Parameters.Add("qt", Quant);

				//
				// executar o comando 
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = ShopStatusEnum.NOT_OK;
				else
					// actualizou registo
					status = ShopStatusEnum.OK;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <AddStock> class 'Product': " + ex.Message);
				status = ShopStatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}
		#endregion

		#region mtodos internos

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem.
		/// este mtodo no faz parte do interface pblico da classe, apenas  
		/// utilizado internamente na ClassLibrary
		/// </summary>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
		internal ShopStatusEnum InStock(OleDbConnection conn, OleDbTransaction tx, int ProductID, int Quant)
		{
			//
			// efectuar o select
			//
			DataSet rs = UtilDB.GetByID(conn, tx, "Products", "ProductID", ProductID);
			if (rs == null)
				return ShopStatusEnum.NOT_OK;

			//
			// validar stock
			//
			if (rs.Tables["products"].Rows.Count > 0)
				return ((int)(rs.Tables["Products"].Rows[0]["StockQtd"]) >= Quant ? ShopStatusEnum.OK : ShopStatusEnum.INSUFICIENT_STOCK);
			else
				return ShopStatusEnum.INVALID_PRODUCT_ID;
		}

		#endregion
	}
}
